@@ -83,7 +83,7 @@ class FMRadioUI(Component, Window):
         self.__sound_pipe = SoundPipe()
         self.__use_speaker = False
         self.__is_interrupted = False
-        self.__was_playing_before_interruption = False
+        #self.__was_playing_before_interruption = False
         self.__have_headphones = False
         
         self.__stations = config.get_stations()
@@ -129,7 +129,9 @@ class FMRadioUI(Component, Window):
 
     def __init_app(self):
     
-        self.__radio_on()
+        if (self.__assert_headphones()):
+            self.__radio_on()
+        
         if (self.__radio):
             self.__set_frequency(config.get_lastfreq())
             self.__display_note("The Bluetooth module is responsible for\n" \
@@ -608,11 +610,11 @@ class FMRadioUI(Component, Window):
         self.__is_interrupted = True
         
         if (self.__radio):
-            self.__was_playing_before_interruption = True
+            #self.__was_playing_before_interruption = True
             self.__display_note("FM Radio Interrupted")
             
-        else:
-            self.__was_playing_before_interruption = False
+        #else:
+        #    self.__was_playing_before_interruption = False
 
         self.__radio_off()
 
@@ -625,8 +627,8 @@ class FMRadioUI(Component, Window):
         if (not self.__is_interrupted or not self.__have_headphones): return
         self.__is_interrupted = False
         
-        if (self.__was_playing_before_interruption):
-            self.__radio_on()
+        #if (self.__was_playing_before_interruption):
+        self.__radio_on()
  
 
     def handle_CORE_EV_FOLDER_INVALIDATED(self, f):
