@@ -4,9 +4,6 @@ from theme import theme
 
 class StationItem(Item):
 
-    EVENT_SELECTED = "event-selected"
-    
-
     def __init__(self, name, frequency):
     
         self.__name = name
@@ -15,11 +12,6 @@ class StationItem(Item):
         Item.__init__(self)
         
         
-    def connect_selected(self, cb, *args):
-    
-        self._connect(self.EVENT_SELECTED, cb, *args)
-        
-        
     def get_frequency(self):
     
         return self.__frequency
@@ -34,7 +26,7 @@ class StationItem(Item):
             pmap.fill_area(0, 0, w, h, theme.color_mb_background)
 
             # render selection frame
-            if (self.is_hilighted()):
+            if (self.is_hilighted() or self.is_marked()):
                 pmap.draw_frame(theme.mb_selection_frame, 0, 0, w, h, True,
                                 pmap.TOP | pmap.BOTTOM | pmap.LEFT | pmap.RIGHT)
            
@@ -55,9 +47,4 @@ class StationItem(Item):
         
         # copy to the given canvas
         cnv.copy_buffer(pmap, 0, 0, x, y, w, h)
-        
-        
-    def click_at(self, px, py):
-    
-        self.emit_event(self.EVENT_SELECTED, self.__frequency)
 
