@@ -13,7 +13,7 @@ import time
 
 _FPS = 50
 _SCROLL_DELAY = 1.0 / _FPS
-_DRAG_THRESHOLD = 5
+_DRAG_THRESHOLD = 10
 
 # a click must last at least this long to be recognized as such
 _CLICK_THRESHOLD = 0
@@ -54,6 +54,10 @@ class KineticScroller(EventEmitter, Observable):
         self.__drag_pointer = (0, 0)
         self.__drag_begin = 0
         self.__pointer = (0, 0)
+        self.__drag_threshold = _DRAG_THRESHOLD
+        
+        # area allowed for dragging, None for whole widget
+        self.__drag_area = None
                 
         # whether kinetic scrolling is enabled
         self.__kinetic_enabled = True
@@ -100,6 +104,21 @@ class KineticScroller(EventEmitter, Observable):
         self._connect(self.EVENT_TAP_AND_HOLD, cb, *args)
 
 
+    def set_drag_threshold(self, t):
+        """
+        Sets the threshold amount for registering motions as dragging.
+        
+        @param t: threshold in pixels
+        """
+    
+        self.__drag_threshold = t
+
+
+    def set_drag_area(self, x, y, w, h):
+    
+        self.__drag_area = (x, y, w, h)
+
+
     def impulse(self, force_x, force_y):
     
         self.__delta_s = (force_x, force_y)
@@ -195,6 +214,12 @@ class KineticScroller(EventEmitter, Observable):
         
     def __on_button_pressed(self, px, py):
     
+        if (self.__drag_area):
+            ax, ay, aw, ah = self.__drag_area
+            if (px < ax or px > ax + aw or py < ay or py > ay + ah):
+                return
+        #end if
+    
         self.__pointer = (px, py)
         self.__is_button_pressed = True
 
@@ -246,8 +271,8 @@ class KineticScroller(EventEmitter, Observable):
 
 
         # begin dragging
-        if (not self.__is_dragging and (abs(dx) > _DRAG_THRESHOLD or \
-                                        abs(dy) > _DRAG_THRESHOLD)):
+        if (not self.__is_dragging and (abs(dx) > self.__drag_threshold or \
+                                        abs(dy) > self.__drag_threshold)):
             self.__abort_tap_and_hold()
             self.__is_click = False
             self.__is_dragging = True
