@@ -18,8 +18,8 @@ class Widget(EventEmitter):
     EVENT_BUTTON_PRESS = "button-pressed"
     EVENT_BUTTON_RELEASE = "button-released"
     EVENT_MOTION = "motion"
-    EVENT_KEY_PRESS = "key-pressed"
-    EVENT_KEY_RELEASE = "key-released"
+    EVENT_KEY_PRESSED = "key-pressed"
+    EVENT_KEY_RELEASED = "key-released"
     
     SLIDE_LEFT = 0
     SLIDE_RIGHT = 1
@@ -33,7 +33,7 @@ class Widget(EventEmitter):
     __instances = []
 
     # event zones; table: ident -> (window, x, y, w, h, tstamp, cb)
-    _zones = {}
+    #_zones = {}
 
         
 
@@ -44,13 +44,16 @@ class Widget(EventEmitter):
         """
     
         # known actors
-        self.__actors_stack = []
+        #self.__actors_stack = []
+    
+        # younger widgets have precedence over older ones in event handling
+        #self.__age_tstamp = 0
     
         self.__children = []
         self.__parent = None
     
         self.__locked_zone = None
-        self.__need_to_check_zones = False
+        #self.__need_to_check_zones = False
         
         self.__is_enabled = True
         self.__is_frozen = False
@@ -62,6 +65,9 @@ class Widget(EventEmitter):
         
         self.__position = (0, 0)
         self.__size = (0, 0)
+
+        # render overlay handlers
+        self.__overlayers = []
         
         self.__clip_rect = None
         
@@ -71,15 +77,16 @@ class Widget(EventEmitter):
 
         EventEmitter.__init__(self)
 
-          
+
+    """
     def push_actor(self, w):
-        """
+        ""
         Pushes an actor widget onto the actors stack. All but the topmost actor
         on the stack are frozen.
         @since: 0.96.5
         
         @param w: a widget
-        """
+        ""
         
         if (self.__parent):
             self.__parent.push_actor(w)
@@ -89,10 +96,10 @@ class Widget(EventEmitter):
         
         
     def pop_actor(self):
-        """
+        ""
         Pops the topmost actor widget from the actors stack.
         @since: 0.96.5
-        """
+        ""
         
         if (self.__parent):
             self.__parent.pop_actor()
@@ -109,7 +116,8 @@ class Widget(EventEmitter):
                 actor.set_frozen(True)
 
             self.__actors_stack[-1].set_frozen(False)
-            
+    """
+   
             
     def grab_focus(self):
         """
@@ -119,26 +127,58 @@ class Widget(EventEmitter):
         win = self.get_window()
         win._input_focus_widget = self
         
+        
+    def _find_zone_at(self, px, py, ev_name):
+    
+        #print "CHECKING", self
+        if (self.is_frozen() or not self.is_visible()):
+            return None
+            
+        x, y = self.get_screen_pos()
+        w, h = self.get_size()
+        if (not x <= px <= x + w or not y <= py <= y + h):
+            return None
+            
+        #print "CHILDREN", self.__children
+        for i in range(len(self.__children) - 1, -1, -1):
+            c = self.__children[i]
+            zone = c._find_zone_at(px, py, ev_name)
+            #print "CHILD", c, zone
+            if (zone):
+                return zone
+        #end for
+        
+        if (self.has_events): #(ev_name)):
+            return self.__on_action
+        else:
+            return None
+        
+        
           
     def _handle_event(self, ev, px, py, *args):
     
         zone = None
-        zone_tstamp = 0
+        #zone_tstamp = -1
         
         if (ev == Widget.EVENT_BUTTON_PRESS):
-            for w in self._zones:
-                if (w.is_frozen()):
+            zone = self._find_zone_at(px, py, ev)
+            #print "ZONE", px, py, zone
+                
+            """
+            for wdgt in self._zones:
+                if (wdgt.is_frozen()):
                     continue
-                #print self._zones[w]
-                window, x, y, w, h, tstamp, cb = self._zones[w]
+                
+                window, x, y, w, h, tstamp, cb = self._zones[wdgt]
 
                 if (window != self.get_window()):
                     continue
-            #for x, y, w, h, tstamp, cb in self._zones.values():
+
                 if (x <= px <= x + w and y <= py <= y + h and tstamp > zone_tstamp):
                     zone = cb
                     zone_tstamp = tstamp
             #end for
+            """
         
             if (zone):            
                 self.__locked_zone = zone
@@ -169,6 +209,9 @@ class Widget(EventEmitter):
                         event type)
         """
        
+        self.emit_event(ev, *args)
+
+        """
         if (ev in (self.EVENT_KEY_PRESS, self.EVENT_KEY_RELEASE) and
               self._input_focus_widget and
               self._input_focus_widget.is_visible()):
@@ -176,6 +219,7 @@ class Widget(EventEmitter):
         
         else:
             self.emit_event(ev, *args)
+        """
         
         
     def __on_action(self, etype, px, py):
@@ -209,6 +253,11 @@ class Widget(EventEmitter):
         
         return self.__animation_lock.isSet()
 
+    
+    def add_overlayer(self, overlayer):
+    
+        self.__overlayers.append(overlayer)
+
 
     def get_children(self):
         """
@@ -229,7 +278,7 @@ class Widget(EventEmitter):
         self.__children.append(child)
         child.set_parent(self)
         child.set_screen(self.get_screen())
-        self.__check_zone()
+        #self.__check_zone()
         
         
     def remove(self, child):
@@ -241,7 +290,7 @@ class Widget(EventEmitter):
     
         self.__children.remove(child)
         child.set_visible(False)
-        self.__check_zone()
+        #self.__check_zone()
         
         
     def set_parent(self, parent):
@@ -253,6 +302,7 @@ class Widget(EventEmitter):
         """
     
         self.__parent = parent
+        self.__age_tstamp = time.time()
         
         
     def get_parent(self):
@@ -320,7 +370,7 @@ class Widget(EventEmitter):
         """
     
         #print "ZONE", x, y, w, h, ident, self.get_window()
-        self._zones[ident] = (self.get_window(), x, y, w, h, time.time(), self.__on_action)
+        self._zones[ident] = (self.get_window(), x, y, w, h, self.__age_tstamp, self.__on_action)
         
         #if (self.__event_sensor):
         #    self.__event_sensor.set_zone(ident, x, y, w, h, time.time(),
@@ -346,7 +396,7 @@ class Widget(EventEmitter):
         
         self.__is_enabled = value
         #print self, value
-        self.__check_zone()
+        #self.__check_zone()
         
         for c in self.__children:
             c.set_enabled(value)
@@ -409,7 +459,7 @@ class Widget(EventEmitter):
                 f(c)
     
         self.__is_visible = value
-        self.__check_zones()
+        #self.__check_zones()
         f(self)
 
 
@@ -450,7 +500,7 @@ class Widget(EventEmitter):
     def _set_can_be_visible(self, value):
     
         self.__can_be_visible = value
-        self.__check_zone()
+        #self.__check_zone()
                 
         for c in self.__children:
             c._set_can_be_visible(value)
@@ -473,6 +523,7 @@ class Widget(EventEmitter):
         return (self.__screen and self.is_visible() and not self.is_frozen())
         
 
+    """
     def __check_zone(self):
     
         # don't check zone when widget does not have event handlers
@@ -493,16 +544,20 @@ class Widget(EventEmitter):
         self.__check_zone()
         for c in self.__children:
             c.__check_zones()
+    """
+
 
         
+    """
     def _connect(self, etype, cb, *args):
-        """
+        ""
         Connects a callback to an event type. This is a low-level function
         and should only be used when implementing new widgets.
-        """
+        ""
     
         EventEmitter._connect(self, etype, cb, *args)
         self.__check_zone()
+    """
 
 
     def connect_clicked(self, cb, *args):
@@ -567,7 +622,7 @@ class Widget(EventEmitter):
         @param *args: variable list of user arguments
         """
     
-        self._connect(self.EVENT_KEY_PRESS,
+        self._connect(self.EVENT_KEY_PRESSED,
                       lambda key,*a:cb(key, *a),
                       *args)
                       
@@ -580,7 +635,7 @@ class Widget(EventEmitter):
         @param *args: variable list of user arguments
         """
     
-        self._connect(self.EVENT_KEY_RELEASE,
+        self._connect(self.EVENT_KEY_RELEASED,
                       lambda key,*a:cb(key, *a),
                       *args)
 
@@ -594,7 +649,7 @@ class Widget(EventEmitter):
     
         if ((x, y) != self.__position):
             self.__position = (x, y)
-            self.__need_to_check_zones = True
+            #self.__need_to_check_zones = True
             #self.__check_zones()
         
         
@@ -636,7 +691,8 @@ class Widget(EventEmitter):
     
         if ((w, h) != self.__size):
             self.__size = (w, h)
-            self.__check_zone()
+            #self.__need_to_check_zones = True
+            #self.__check_zone()
         
         
     def get_size(self):
@@ -690,6 +746,20 @@ class Widget(EventEmitter):
         pass
 
 
+    def render_overlays(self, screen):
+        """
+        Invokes the registered overlay handlers on the given screen. The screen
+        is usually an offscreen buffer.
+        """
+        
+        for o in self.__overlayers:
+            try:
+                o(self, screen)
+            except:
+                pass
+        #end for
+
+
     def overlay_this(self):
         """
         Widgets may override this method for overlay effects. This method is
@@ -715,8 +785,8 @@ class Widget(EventEmitter):
             return
         
         logging.debug("rendering widget %s", `self`)
-        if (self.__need_to_check_zones):
-            self.__check_zones()
+        #if (self.__need_to_check_zones):
+        #    self.__check_zones()
 
         self.render_this()
 
@@ -761,7 +831,7 @@ class Widget(EventEmitter):
         self.__parent = parent
         self.set_screen(real_screen)
         self.set_pos(real_x, real_y)
-        self.__check_zones()
+        #self.__check_zones()
         
         
     def skip_next_render(self):
