@@ -55,6 +55,6 @@ class EventEmitter(object):
                 a = args + u_args
                 cb(*a)
             except:
-                pass
+                import traceback; traceback.print_exc()
         #end for
 
